@@ -31,13 +31,6 @@ typedef float float32_t;
 #include <inttypes.h>
 #endif
 
-typedef unsigned char   uchar;
-
-#if !defined(__GNUC__) || defined(WIN32)
-typedef unsigned short  ushort;
-typedef unsigned int    uint;
-#endif
-
 // RIFF / wav header formats (these occur at the beginning of both wav files
 // and pre-4.0 WavPack files that are not in the "raw" mode). Generally, an
 // application using the library to read or write WavPack files will not be
@@ -57,12 +50,12 @@ typedef struct {
 #define ChunkHeaderFormat "4L"
 
 typedef struct {
-    ushort FormatTag, NumChannels;
+    unsigned short FormatTag, NumChannels;
     uint32_t SampleRate, BytesPerSecond;
-    ushort BlockAlign, BitsPerSample;
-    ushort cbSize, ValidBitsPerSample;
+    unsigned short BlockAlign, BitsPerSample;
+    unsigned short cbSize, ValidBitsPerSample;
     int32_t ChannelMask;
-    ushort SubFormat;
+    unsigned short SubFormat;
     char GUID [14];
 } WaveHeader;
 
@@ -79,7 +72,7 @@ typedef struct {
     char ckID [4];
     uint32_t ckSize;
     short version;
-    uchar track_no, index_no;
+    unsigned char track_no, index_no;
     uint32_t total_samples, block_index, block_samples, flags, crc;
 } WavpackHeader;
 
@@ -164,7 +157,7 @@ typedef struct {
     int bits_per_sample, bytes_per_sample;
     int qmode, flags, xmode, num_channels, float_norm_exp;
     int32_t block_samples, extra_flags, sample_rate, channel_mask;
-    uchar md5_checksum [16], md5_read;
+    unsigned char md5_checksum [16], md5_read;
     int num_tag_strings;
     char **tag_strings;
 } WavpackConfig;
@@ -262,9 +255,9 @@ int WavpackGetNumChannels (WavpackContext *wpc);
 int WavpackGetChannelMask (WavpackContext *wpc);
 int WavpackGetReducedChannels (WavpackContext *wpc);
 int WavpackGetFloatNormExp (WavpackContext *wpc);
-int WavpackGetMD5Sum (WavpackContext *wpc, uchar data [16]);
+int WavpackGetMD5Sum (WavpackContext *wpc, unsigned char data [16]);
 uint32_t WavpackGetWrapperBytes (WavpackContext *wpc);
-uchar *WavpackGetWrapperData (WavpackContext *wpc);
+unsigned char *WavpackGetWrapperData (WavpackContext *wpc);
 void WavpackFreeWrapper (WavpackContext *wpc);
 void WavpackSeekTrailingWrapper (WavpackContext *wpc);
 double WavpackGetProgress (WavpackContext *wpc);
@@ -286,7 +279,7 @@ int WavpackWriteTag (WavpackContext *wpc);
 WavpackContext *WavpackOpenFileOutput (WavpackBlockOutput blockout, void *wv_id, void *wvc_id);
 int WavpackSetConfiguration (WavpackContext *wpc, WavpackConfig *config, uint32_t total_samples);
 int WavpackAddWrapper (WavpackContext *wpc, void *data, uint32_t bcount);
-int WavpackStoreMD5Sum (WavpackContext *wpc, uchar data [16]);
+int WavpackStoreMD5Sum (WavpackContext *wpc, unsigned char data [16]);
 int WavpackPackInit (WavpackContext *wpc);
 int WavpackPackSamples (WavpackContext *wpc, int32_t *sample_buffer, uint32_t sample_count);
 int WavpackFlushSamples (WavpackContext *wpc);
