@@ -55,6 +55,7 @@ WavpackContext *open_file3 (WavpackContext *wpc, char *error)
     WavpackStream3 *wps;
     WaveHeader3 wavhdr;
 
+    CLEAR (wavhdr);
     wpc->stream3 = wps = (WavpackStream3 *) malloc (sizeof (WavpackStream3));
     CLEAR (*wps);
 
@@ -110,34 +111,40 @@ WavpackContext *open_file3 (WavpackContext *wpc, char *error)
                     if (ChunkHeader.ckSize > sizeof (wavhdr)) {
                         uint32_t bytes_to_skip = (ChunkHeader.ckSize + 1 - sizeof (wavhdr)) & ~1L;
 
+                        if (bytes_to_skip > 1024 * 1024) {
+                            strcpy (error, "not a valid WavPack file!");
+                            return WavpackCloseFile (wpc);
+                        }
+
                         if (wpc->open_flags & OPEN_WRAPPER) {
                             wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bytes_to_skip);
                             wpc->reader->read_bytes (wpc->wv_in, wpc->wrapper_data + wpc->wrapper_bytes, bytes_to_skip);
                             wpc->wrapper_bytes += bytes_to_skip;
                         }
                         else {
-                            uchar *temp = malloc (bytes_to_skip);
+                            unsigned char *temp = malloc (bytes_to_skip);
                             wpc->reader->read_bytes (wpc->wv_in, temp, bytes_to_skip);
                             free (temp);
                         }
                     }
                 }
-                else if (!strncmp (ChunkHeader.ckID, "data", 4)) {
-                    wpc->total_samples = ChunkHeader.ckSize / wavhdr.NumChannels /
-                        ((wavhdr.BitsPerSample > 16) ? 3 : 2);
-
+                else if (!strncmp (ChunkHeader.ckID, "data", 4))
                     break;
-                }
                 else if ((ChunkHeader.ckSize + 1) & ~1L) {
                     uint32_t bytes_to_skip = (ChunkHeader.ckSize + 1) & ~1L;
 
+                    if (bytes_to_skip > 1024 * 1024) {
+                        strcpy (error, "not a valid WavPack file!");
+                        return WavpackCloseFile (wpc);
+                    }
+
                     if (wpc->open_flags & OPEN_WRAPPER) {
                         wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bytes_to_skip);
                         wpc->reader->read_bytes (wpc->wv_in, wpc->wrapper_data + wpc->wrapper_bytes, bytes_to_skip);
                         wpc->wrapper_bytes += bytes_to_skip;
                     }
                     else {
-                        uchar *temp = malloc (bytes_to_skip);
+                        unsigned char *temp = malloc (bytes_to_skip);
                         wpc->reader->read_bytes (wpc->wv_in, temp, bytes_to_skip);
                         free (temp);
                     }
@@ -150,6 +157,17 @@ WavpackContext *open_file3 (WavpackContext *wpc, char *error)
         return WavpackCloseFile (wpc);
     }
 
+    if (wavhdr.FormatTag != 1 || !wavhdr.NumChannels || wavhdr.NumChannels > 2 ||
+        !wavhdr.SampleRate || wavhdr.BitsPerSample < 16 || wavhdr.BitsPerSample > 24 ||
+        wavhdr.BlockAlign / wavhdr.NumChannels > 3 || wavhdr.BlockAlign % wavhdr.NumChannels ||
+        wavhdr.BlockAlign / wavhdr.NumChannels < (wavhdr.BitsPerSample + 7) / 8) {
+            strcpy (error, "not a valid WavPack file!");
+            return WavpackCloseFile (wpc);
+    }
+
+    wpc->total_samples = ChunkHeader.ckSize / wavhdr.NumChannels /
+        ((wavhdr.BitsPerSample > 16) ? 3 : 2);
+
     if (wpc->reader->read_bytes (wpc->wv_in, &wphdr, 10) != 10) {
         strcpy (error, "not a valid WavPack file!");
         return WavpackCloseFile (wpc);
@@ -309,7 +327,7 @@ static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id)
     bs->fpos = (bs->reader = reader)->get_pos (bs->id = id);
 
     if (!bs->buf)
-        bs->buf = (uchar *) malloc (bs->bufsiz);
+        bs->buf = (unsigned char *) malloc (bs->bufsiz);
 
     bs->end = bs->buf + bs->bufsiz;
     bs->ptr = bs->end - 1;
@@ -326,7 +344,7 @@ static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id)
 // to be read from the file. This function is NOT supported for overlapped
 // operation.
 
-void bs_restore3 (Bitstream3 *bs)
+static void bs_restore3 (Bitstream3 *bs)
 {
     uint32_t bytes_to_read = (uint32_t)(bs->end - bs->ptr - 1), bytes_read;
 
@@ -364,7 +382,7 @@ static uint32_t bs_unused_bytes (Bitstream3 *bs)
     return (uint32_t)(bs->end - bs->ptr);
 }
 
-static uchar *bs_unused_data (Bitstream3 *bs)
+static unsigned char *bs_unused_data (Bitstream3 *bs)
 {
     if (bs->bc < 8) {
         bs->bc += 8;
@@ -600,7 +618,7 @@ static void *unpack_restore (WavpackStream3 *wps, void *source, int keep_resourc
     int flags = wps->wphdr.flags, tcount;
     struct decorr_pass *dpp;
     FILE *temp_file;
-    uchar *temp_buf;
+    unsigned char *temp_buf;
 
     unpack_init3 (wps);
     temp_file = wps->wvbits.id;
@@ -787,7 +805,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
     int32_t read_word, *bptr;
     int32_t sample [2] [2];
     int weight [2] [1];
-    uint i;
+    unsigned int i;
 
 #ifdef ATTEMPT_ERROR_MUTING
     int32_t mute_limit = (flags & BYTES_3) ? 8388608 : 32768;
@@ -820,7 +838,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
     if (!wps->index_points [points_index].saved) {
 
         if (!wps->unpack_data)
-            wps->unpack_data = (uchar *) malloc (256 * (wps->unpack_size = unpack_size (wps)));
+            wps->unpack_data = (unsigned char *) malloc (256 * (wps->unpack_size = unpack_size (wps)));
 
         wps->index_points [points_index].sample_index = wps->sample_index;
         unpack_save (wps, wps->unpack_data + points_index * wps->unpack_size);
@@ -1687,7 +1705,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
             wpc->crc_errors++;
 
         if (wpc->open_flags & OPEN_WRAPPER) {
-            uchar *temp = malloc (1024);
+            unsigned char *temp = malloc (1024);
             uint32_t bcount;
 
             if (bs_unused_bytes (&wps->wvbits)) {
@@ -1772,7 +1790,7 @@ static void init_words3 (WavpackStream3 *wps)
 static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
 {
     uint32_t tmp1, tmp2, avalue;
-    uint ones_count;
+    unsigned int ones_count;
     int k;
 
     if ((wps->wphdr.flags & EXTREME_DECORR) && !(wps->wphdr.flags & OVER_20)) {
@@ -1814,6 +1832,9 @@ static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
     k = (wps->w1.ave_level [0] [chan] + (wps->w1.ave_level [0] [chan] >> 3) + 0x40) >> 7;
     k = count_bits (k);
 
+    if (k & ~31)
+        return WORD_EOF;
+
     if (ones_count == 0) {
         getbits (&avalue, k, &wps->wvbits);
         avalue &= bitmask [k];
@@ -1823,6 +1844,9 @@ static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
         k = (wps->w1.ave_level [1] [chan] + (wps->w1.ave_level [1] [chan] >> 4) + 0x20) >> 6;
         k = count_bits (k);
 
+        if (k & ~31)
+            return WORD_EOF;
+
         if (ones_count == 1) {
             getbits (&avalue, k, &wps->wvbits);
             avalue &= bitmask [k];
@@ -1839,6 +1863,10 @@ static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
             else {
                 k = (wps->w1.ave_level [2] [chan] + 0x10) >> 5;
                 k = count_bits (k);
+
+                if (k & ~31)
+                    return WORD_EOF;
+
                 getbits (&avalue, k, &wps->wvbits);
                 avalue = (avalue & bitmask [k]) + (bitset [k] * (ones_count - 2));
             }
@@ -1864,7 +1892,7 @@ static int32_t FASTCALL get_word1 (WavpackStream3 *wps, int chan)
 static int32_t FASTCALL get_old_word1 (WavpackStream3 *wps, int chan)
 {
     uint32_t avalue;
-    uint bc;
+    unsigned int bc;
     int k;
 
     if (!wps->w1.index [chan]) {
@@ -1893,7 +1921,7 @@ static int32_t FASTCALL get_old_word1 (WavpackStream3 *wps, int chan)
 
     for (bc = 0; bc < 32 && getbit (&wps->wvbits); ++bc);
 
-    if (bc == 32)
+    if (bc == 32 || (k & ~31))
         return WORD_EOF;
 
     avalue = (avalue & bitmask [k]) + bitset [k] * bc;
